(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common'), require('@angular/cdk/overlay'), require('@angular/cdk/portal'), require('rxjs/observable/fromEvent'), require('rxjs/observable/merge')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/common', '@angular/cdk/overlay', '@angular/cdk/portal', 'rxjs/observable/fromEvent', 'rxjs/observable/merge'], factory) :
	(factory((global['ng2-right-click-menu'] = {}),global.ng.core,global.ng.common,global.ng.cdk.overlay,global.ng.cdk.portal,global.Rx.Observable,global.Rx.Observable));
}(this, (function (exports,core,common,overlay,portal,fromEvent,merge) { 'use strict';

var MenuItemContext = /** @class */ (function () {
    function MenuItemContext() {
        this.$implicit = {};
    }
    return MenuItemContext;
}());
var ShContextMenuItemDirective = /** @class */ (function () {
    /**
     * @param {?} template
     */
    function ShContextMenuItemDirective(template) {
        this.template = template;
        this.closeOnClick = true;
        this.click = new core.EventEmitter();
        this.context = new MenuItemContext();
    }
    /**
     * @return {?}
     */
    ShContextMenuItemDirective.prototype.setNotActive = function () {
        this._active = false;
        if (this.subMenu) {
            this.subMenu.setNotActive();
        }
    };
    /**
     * @return {?}
     */
    ShContextMenuItemDirective.prototype.setActive = function () {
        this._active = true;
    };
    return ShContextMenuItemDirective;
}());
ShContextMenuItemDirective.decorators = [
    { type: core.Directive, args: [{
                selector: '[shContextMenuItem]'
            },] },
];
/**
 * @nocollapse
 */
ShContextMenuItemDirective.ctorParameters = function () { return [
    { type: core.TemplateRef, decorators: [{ type: core.Optional },] },
]; };
ShContextMenuItemDirective.propDecorators = {
    'subMenu': [{ type: core.Input },],
    'divider': [{ type: core.Input },],
    'visible': [{ type: core.Input },],
    'disabled': [{ type: core.Input },],
    'closeOnClick': [{ type: core.Input },],
    'click': [{ type: core.Output },],
};
var ShContextMenuService = /** @class */ (function () {
    /**
     * @param {?} overlay
     */
    function ShContextMenuService(overlay$$1) {
        this.overlay = overlay$$1;
        this.activeOverlays = [];
    }
    /**
     * @param {?} ctxEvent
     * @return {?}
     */
    ShContextMenuService.prototype.openMenu = function (ctxEvent) {
        this.closeCurrentOverlays();
        var menu = ctxEvent.menu, mouseEvent = ctxEvent.mouseEvent, targetElement = ctxEvent.targetElement, data = ctxEvent.data;
        this.activeMenu = menu;
        mouseEvent.preventDefault();
        mouseEvent.stopPropagation();
        this.overrideGetBoundingClientRect(targetElement, mouseEvent);
        var /** @type {?} */ scrollStrategy = this.buildCloseScrollStrategy();
        var /** @type {?} */ positionStrategy = this.buildConnectedPositionStrategy(targetElement);
        this.attachContextToItems(menu, data);
        var /** @type {?} */ overlayRef = this.createAndAttachOverlay(positionStrategy, scrollStrategy, menu, true);
        this.attachOverlayRef(menu, overlayRef);
        this.registerBackdropEvents(overlayRef);
    };
    /**
     * @param {?} ctxEvent
     * @return {?}
     */
    ShContextMenuService.prototype.openSubMenu = function (ctxEvent) {
        var menu = ctxEvent.menu, mouseEvent = ctxEvent.mouseEvent, targetElement = ctxEvent.targetElement, data = ctxEvent.data, parentMenu = ctxEvent.parentMenu;
        mouseEvent.preventDefault();
        mouseEvent.stopPropagation();
        var /** @type {?} */ scrollStrategy = this.buildCloseScrollStrategy();
        var /** @type {?} */ positionStrategy = this.buildConnectedPositionStrategyForSubMenu(targetElement);
        var /** @type {?} */ overlayRef = this.createAndAttachOverlay(positionStrategy, scrollStrategy, menu, false);
        this.attachContextToItems(menu, data);
        this.attachThisContext(menu, parentMenu);
        this.attachOverlayRef(menu, overlayRef);
    };
    /**
     * @return {?}
     */
    ShContextMenuService.prototype.destroy = function () {
        this.closeCurrentOverlays();
        this.backDropSub.unsubscribe();
    };
    /**
     * @return {?}
     */
    ShContextMenuService.prototype.ngOnDestroy = function () {
        this.destroy();
    };
    /**
     * @param {?} menu
     * @return {?}
     */
    ShContextMenuService.prototype.closeSubMenus = function (menu) {
        var _this = this;
        var /** @type {?} */ itemsWithSubMenus = menu
            .menuItems
            .filter(function (i) { return !!i.subMenu && !!i.subMenu.overlayRef; });
        if (itemsWithSubMenus.length) {
            itemsWithSubMenus.forEach(function (sm) { return _this.closeSubMenus(sm.subMenu); });
            var /** @type {?} */ overlayRefs = itemsWithSubMenus
                .map(function (i) { return i.subMenu.overlayRef; });
            overlayRefs.forEach(function (r) { return r.dispose(); });
        }
    };
    /**
     * @param {?} overlayRef
     * @return {?}
     */
    ShContextMenuService.prototype.registerBackdropEvents = function (overlayRef) {
        var /** @type {?} */ elm = overlayRef.backdropElement;
        this.backDropSub = fromEvent.fromEvent(elm, 'mousedown')
            .subscribe(this.closeCurrentOverlays.bind(this));
    };
    /**
     * @param {?} positionStrategy
     * @param {?} scrollStrategy
     * @param {?} menu
     * @param {?=} hasBackdrop
     * @return {?}
     */
    ShContextMenuService.prototype.createAndAttachOverlay = function (positionStrategy, scrollStrategy, menu, hasBackdrop) {
        if (hasBackdrop === void 0) { hasBackdrop = true; }
        var /** @type {?} */ overlayRef = this.overlay.create({
            positionStrategy: positionStrategy,
            scrollStrategy: scrollStrategy,
            hasBackdrop: hasBackdrop,
            backdropClass: 'sh-backdrop'
        });
        /*
         TODO: try passing the TemplatePortal context (data)
         and then injecting it to the *ngTemplateOutlet in the component template
        */
        var /** @type {?} */ menuPortal = new portal.TemplatePortal(menu.menuTemplate, menu.menuContainer);
        overlayRef.attach(menuPortal);
        this.activeOverlays.push(overlayRef);
        return overlayRef;
    };
    /**
     * @return {?}
     */
    ShContextMenuService.prototype.buildCloseScrollStrategy = function () {
        return this.overlay.scrollStrategies.close();
    };
    /**
     * @param {?} elm
     * @return {?}
     */
    ShContextMenuService.prototype.buildConnectedPositionStrategy = function (elm) {
        return this
            .overlay
            .position()
            .connectedTo(elm, { originX: 'start', originY: 'bottom' }, { overlayX: 'start', overlayY: 'top' })
            .withFallbackPosition({ originX: 'start', originY: 'top' }, { overlayX: 'start', overlayY: 'bottom' })
            .withFallbackPosition({ originX: 'end', originY: 'top' }, { overlayX: 'start', overlayY: 'top' })
            .withFallbackPosition({ originX: 'start', originY: 'top' }, { overlayX: 'end', overlayY: 'top' })
            .withFallbackPosition({ originX: 'end', originY: 'center' }, { overlayX: 'start', overlayY: 'center' })
            .withFallbackPosition({ originX: 'start', originY: 'center' }, { overlayX: 'end', overlayY: 'center' });
    };
    /**
     * @param {?} elm
     * @return {?}
     */
    ShContextMenuService.prototype.buildConnectedPositionStrategyForSubMenu = function (elm) {
        return this
            .overlay
            .position()
            .connectedTo(elm, { originX: 'end', originY: 'top' }, { overlayX: 'start', overlayY: 'top' })
            .withFallbackPosition({ originX: 'start', originY: 'top' }, { overlayX: 'end', overlayY: 'top' })
            .withFallbackPosition({ originX: 'end', originY: 'bottom' }, { overlayX: 'start', overlayY: 'bottom' })
            .withFallbackPosition({ originX: 'start', originY: 'bottom' }, { overlayX: 'end', overlayY: 'bottom' });
    };
    /**
     * @param {?} elm
     * @param {?} event
     * @return {?}
     */
    ShContextMenuService.prototype.overrideGetBoundingClientRect = function (elm, event) {
        var clientX = event.clientX, clientY = event.clientY;
        elm.nativeElement.getBoundingClientRect = function () {
            return {
                bottom: clientY,
                height: 0,
                left: clientX,
                right: clientX,
                top: clientY,
                width: 0
            };
        };
    };
    /**
     * @return {?}
     */
    ShContextMenuService.prototype.closeCurrentOverlays = function () {
        this.activeOverlays.forEach(function (o) {
            o.detach();
            o.dispose();
        });
        this.activeOverlays = [];
        // TODO: create close subject and emit.
        // subscribe in component
        if (this.activeMenu) {
            this.activeMenu.close();
        }
    };
    /**
     * @param {?} menu
     * @param {?} data
     * @return {?}
     */
    ShContextMenuService.prototype.attachContextToItems = function (menu, data) {
        menu.menuItems.forEach(function (i) { return i.context.$implicit = data; });
    };
    /**
     * @param {?} menu
     * @param {?} parentMenu
     * @return {?}
     */
    ShContextMenuService.prototype.attachThisContext = function (menu, parentMenu) {
        menu.thisContext = parentMenu.thisContext;
    };
    /**
     * @param {?} menu
     * @param {?} overlayRef
     * @return {?}
     */
    ShContextMenuService.prototype.attachOverlayRef = function (menu, overlayRef) {
        menu.overlayRef = overlayRef;
    };
    return ShContextMenuService;
}());
ShContextMenuService.decorators = [
    { type: core.Injectable },
];
/**
 * @nocollapse
 */
ShContextMenuService.ctorParameters = function () { return [
    { type: overlay.Overlay, },
]; };
var ShContextMenuComponent = /** @class */ (function () {
    /**
     * @param {?} ctxService
     */
    function ShContextMenuComponent(ctxService) {
        this.ctxService = ctxService;
        this.contentChildrenItems = new core.QueryList();
        this.viewChildrenItems = new core.QueryList();
    }
    Object.defineProperty(ShContextMenuComponent.prototype, "menuItems", {
        /**
         * @return {?}
         */
        get: function () {
            // when using the ShContextMenuComponent as menu, the ContentChildren is the source
            if (this.contentChildrenItems.length) {
                return this.contentChildrenItems;
            }
            // when using a custom component as menu the ViewChildren is the source
            return this.viewChildrenItems;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} $event
     * @param {?} item
     * @param {?} elm
     * @return {?}
     */
    ShContextMenuComponent.prototype.onEnter = function ($event, item, elm) {
        this.ctxService.closeSubMenus(this);
        this.setNotActive();
        if (!item.subMenu) {
            return;
        }
        this.setActive(item);
        this.ctxService.openSubMenu({
            data: item.context.$implicit,
            targetElement: new core.ElementRef(elm),
            menu: item.subMenu,
            mouseEvent: $event,
            parentMenu: this
        });
    };
    /**
     * @param {?} item
     * @return {?}
     */
    ShContextMenuComponent.prototype.setActive = function (item) {
        item.setActive();
        this.subActive = true;
    };
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    ShContextMenuComponent.prototype.onClick = function (event, item) {
        // TODO: move click handling to service
        if (item.divider) {
            return;
        }
        if (!item.subMenu && item.closeOnClick) {
            this.ctxService.destroy();
            item.click.emit({
                data: item.context.$implicit,
                event: event
            });
        }
    };
    /**
     * @param {?} fn
     * @param {?} fallbackContext
     * @param {?} data
     * @param {?} event
     * @return {?}
     */
    ShContextMenuComponent.prototype.callWithContext = function (fn, fallbackContext, data, event) {
        return fn.call(this.thisContext ? this.thisContext : fallbackContext, { data: data, event: event });
    };
    /**
     * @return {?}
     */
    ShContextMenuComponent.prototype.close = function () {
        this.setNotActive();
        this.menuContainer.detach();
        this.overlayRef.detach();
    };
    /**
     * @return {?}
     */
    ShContextMenuComponent.prototype.ngOnDestroy = function () {
        this.close();
    };
    /**
     * @return {?}
     */
    ShContextMenuComponent.prototype.setNotActive = function () {
        this.subActive = false;
        this.menuItems.forEach(function (i) { return i.setNotActive(); });
    };
    /**
     * @param {?} item
     * @return {?}
     */
    ShContextMenuComponent.prototype.isVisible = function (item) {
        if (!item.visible) {
            return true;
        }
        return this.callWithContext(item.visible, this, item.context.$implicit, null);
    };
    return ShContextMenuComponent;
}());
ShContextMenuComponent.decorators = [
    { type: core.Component, args: [{
                selector: 'sh-context-menu',
                encapsulation: core.ViewEncapsulation.None,
                styles: ["\n    .sh-backdrop {\n      background-color: transparent;\n    }\n\n    .sh-context-menu {\n      background: #ececec;\n      min-width: 150px;\n      border: 1px solid rgba(0, 0, 0, 0.2);\n      border-radius: 3px;\n      -webkit-box-shadow: 0 0 10px 2px rgba(0, 0, 0, 0.1);\n              box-shadow: 0 0 10px 2px rgba(0, 0, 0, 0.1);\n      color: black;\n      padding: 5px 0;\n      margin: 0;\n    }\n\n    .sh-context-menu--item {\n      padding: 5px 10px 5px 15px;\n      -webkit-transition: all 0.15s;\n      transition: all 0.15s;\n    }\n\n    .sh-context-menu--item:hover, .sh-context-menu--item__sub-active {\n      background-color: #4b8bec;\n      color: white;\n      cursor: pointer;\n    }\n\n    .sh-context-menu--item.sh-context-menu--item__divider:hover {\n      background-color: #ececec;\n      color: black;\n      cursor: default;\n    }\n\n    .sh-context-menu--item__divider {\n      height: 1px;\n      margin: 1px 1px 8px 1px;\n      overflow: hidden;\n      border-bottom: 1px solid #d0d0d0;\n    }\n\n    .sh-context-menu--item.sh-sub-anchor {\n      position: relative;\n      min-width: 160px;\n    }\n\n    .sh-sub-anchor:after {\n      content: \"\";\n      top: 50%;\n      right: 6px;\n      -webkit-transform: translateY(-50%);\n              transform: translateY(-50%);\n      position: absolute;\n      border-top: 6px solid transparent;\n      border-bottom: 6px solid transparent;\n      border-left: 8px solid black;\n    }\n  "],
                template: "\n    <ng-container #menuContainer></ng-container>\n    <ng-template #menuTemplate>\n      <div class=\"sh-context-menu\">\n        <div\n          *ngFor=\"let menuItem of menuItems\"\n          #itemElement\n          [ngClass]=\"{'sh-sub-anchor': menuItem.subMenu,\n           'sh-context-menu--item__divider': menuItem.divider,\n           'sh-context-menu--item__sub-active': subActive && menuItem.active}\"\n          class=\"sh-context-menu--item\"\n          (mouseenter)=\"onEnter($event, menuItem, itemElement)\"\n          (click)=\"onClick($event, menuItem)\">\n          <ng-container *ngIf=\"!menuItem.divider || !isVisible(menuItem)\">\n            <ng-content *ngTemplateOutlet=\"menuItem.template; context: menuItem.context\"></ng-content>\n          </ng-container>\n        </div>\n      </div>\n    </ng-template>\n  "
            },] },
];
/**
 * @nocollapse
 */
ShContextMenuComponent.ctorParameters = function () { return [
    { type: ShContextMenuService, },
]; };
ShContextMenuComponent.propDecorators = {
    'thisContext': [{ type: core.Input, args: ['this',] },],
    'contentChildrenItems': [{ type: core.ContentChildren, args: [ShContextMenuItemDirective, { read: ShContextMenuItemDirective },] },],
    'viewChildrenItems': [{ type: core.ViewChildren, args: [ShContextMenuItemDirective, { read: ShContextMenuItemDirective },] },],
    'menuTemplate': [{ type: core.ViewChild, args: ['menuTemplate', { read: core.TemplateRef },] },],
    'menuContainer': [{ type: core.ViewChild, args: ['menuContainer', { read: core.ViewContainerRef },] },],
};
var ShAttachMenuDirective = /** @class */ (function () {
    /**
     * @param {?} ctxService
     * @param {?} elm
     */
    function ShAttachMenuDirective(ctxService, elm) {
        this.ctxService = ctxService;
        this.elm = elm;
    }
    /**
     * @return {?}
     */
    ShAttachMenuDirective.prototype.ngOnInit = function () {
        this.setupEvents();
    };
    /**
     * @return {?}
     */
    ShAttachMenuDirective.prototype.setupEvents = function () {
        var _this = this;
        var /** @type {?} */ observables = [];
        if (!this.triggers) {
            observables.push(fromEvent.fromEvent(this.elm.nativeElement, 'contextmenu'));
        }
        else {
            this.triggers.forEach(function (t) {
                observables.push(fromEvent.fromEvent(_this.elm.nativeElement, t));
            });
        }
        this.sub = merge.merge.apply(void 0, observables).subscribe(this.openMenu.bind(this));
    };
    /**
     * @param {?} event
     * @return {?}
     */
    ShAttachMenuDirective.prototype.openMenu = function (event) {
        this.ctxService.openMenu({
            menu: this.menu,
            mouseEvent: event,
            targetElement: this.elm,
            data: this.data
        });
    };
    /**
     * @return {?}
     */
    ShAttachMenuDirective.prototype.ngOnDestroy = function () {
        this.sub.unsubscribe();
    };
    return ShAttachMenuDirective;
}());
ShAttachMenuDirective.decorators = [
    { type: core.Directive, args: [{
                selector: '[shAttachMenu]'
            },] },
];
/**
 * @nocollapse
 */
ShAttachMenuDirective.ctorParameters = function () { return [
    { type: ShContextMenuService, },
    { type: core.ElementRef, },
]; };
ShAttachMenuDirective.propDecorators = {
    'menu': [{ type: core.Input, args: ['shAttachMenu',] },],
    'triggers': [{ type: core.Input, args: ['shMenuTriggers',] },],
    'data': [{ type: core.Input, args: ['shMenuData',] },],
};
var ShContextMenuModule = /** @class */ (function () {
    function ShContextMenuModule() {
    }
    return ShContextMenuModule;
}());
ShContextMenuModule.decorators = [
    { type: core.NgModule, args: [{
                declarations: [
                    ShAttachMenuDirective,
                    ShContextMenuComponent,
                    ShContextMenuItemDirective
                ],
                exports: [
                    ShAttachMenuDirective,
                    ShContextMenuComponent,
                    ShContextMenuItemDirective
                ],
                providers: [
                    ShContextMenuService
                ],
                imports: [
                    common.CommonModule,
                    overlay.OverlayModule
                ],
                entryComponents: [
                    ShContextMenuComponent
                ]
            },] },
];
/**
 * @nocollapse
 */
ShContextMenuModule.ctorParameters = function () { return []; };

exports.ShContextMenuModule = ShContextMenuModule;
exports.ShContextMenuComponent = ShContextMenuComponent;
exports.ShAttachMenuDirective = ShAttachMenuDirective;
exports.ShContextMenuItemDirective = ShContextMenuItemDirective;
exports.ShContextMenuService = ShContextMenuService;
exports.ɵa = MenuItemContext;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ng2-right-click-menu.umd.js.map
